require 'adminlock/engine'
module Adminlock

  extend ActiveSupport::Concern

  included do
    if respond_to?(:before_action)
      before_action :check_for_adminlock, except: ['unlock']
    else
      before_filter :check_for_adminlock, except: ['unlock']
    end
  end

  def self.from_config(setting, secrets_or_credentials = :credentials)
    return unless Rails.application.respond_to?(secrets_or_credentials)

    store = Rails.application.public_send(secrets_or_credentials)

    store.adminlock.respond_to?(:fetch) &&
      store.adminlock.fetch(setting, store.public_send("adminlock_#{setting}")) ||
      store.public_send("adminlock_#{setting}") || store.public_send(setting)
  end

  private

  def check_for_adminlock
    return unless respond_to?(:adminlock) && adminlock_codeword
    return if cookies[:adminlock].present? && cookies[:adminlock] == adminlock_codeword.to_s.downcase

    redirect_to adminlock.unlock_path(
      return_to: request.fullpath.split('?adminlock_codeword')[0],
      adminlock_codeword: params[:adminlock_codeword]
    )
  end

  def cookie_lifetime
    @cookie_lifetime ||=
      ENV['COOKIE_LIFETIME_IN_WEEKS'] ||
      ENV['cookie_lifetime_in_weeks'] ||
      Adminlock.from_config(:cookie_lifetime_in_weeks, :secrets) ||
      Adminlock.from_config(:cookie_lifetime_in_weeks)
  end

  def adminlock_codeword
    @adminlock_codeword ||=
      ENV['ADMINLOCK_CODEWORD'] ||
      ENV['adminlock_codeword'] ||
      Adminlock.from_config(:codeword, :secrets) ||
      Adminlock.from_config(:codeword)
  end

  def adminlock_cookie_lifetime
    seconds = (cookie_lifetime.to_f * 1.hour).to_i
    if seconds > 0
      seconds
    else
      5.years
    end
  end
end
